#version 330 core

#define MAX_LIGHT_NUM 6

in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace[MAX_LIGHT_NUM];

uniform vec3 color;
uniform int fLightNum;
uniform vec3 LightPosition_worldspace;
uniform vec3 lightColor[MAX_LIGHT_NUM];
uniform float lightPower[MAX_LIGHT_NUM];

out vec4 out_color;

void calculateLight(int lightIndex, out float cosTheta, out float cosAlpha)
{
	// Normal of the computed fragment, in camera space
	vec3 n = normalize( Normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace[lightIndex] );
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendiular to the triangle -> 0
	//  - light is behind the triangle -> 0
	cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	cosAlpha = clamp( dot( E,R ), 0,1 );
};

void main() {

	float cosTheta;
	float cosAlpha;

	vec3 MaterialDiffuseColor = vec3(color);
	vec3 MaterialSpecularColor = vec3(0.2,0.2,0.2);

	vec3 outColor = vec3(0.2,0.2,0.2) * MaterialDiffuseColor;

	for(int lightIndex = 0; lightIndex < fLightNum; lightIndex++)
	{	
		calculateLight(lightIndex,cosTheta,cosAlpha);

		outColor += 
		// Diffuse : "color" of the object
		MaterialDiffuseColor * lightColor[lightIndex] * lightPower[lightIndex] * cosTheta +
		// Specular : reflective highlight, like a mirror
		MaterialSpecularColor * lightColor[lightIndex] * lightPower[lightIndex] * pow(cosAlpha,5);
	}

	out_color = vec4(outColor,1.0f);
}